// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure.Management.Sql.Models;

namespace Microsoft.Azure.Management.Sql
{
    /// <summary>
    /// Represents all the operations for operating on Azure SQL Recommended
    /// Elastic Pools.  Contains operations to: Retrieve.
    /// </summary>
    public partial interface IRecommendedElasticPoolOperations
    {
        /// <summary>
        /// Returns information about an Azure SQL Recommended Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Recommended
        /// Server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server for which the Azure SQL
        /// Recommended Elastic Pool is.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// The name of the Azure SQL Recommended Elastic Pool to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Recommended Resource
        /// pool request.
        /// </returns>
        Task<RecommendedElasticPoolGetResponse> GetAsync(string resourceGroupName, string serverName, string recommendedElasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Recommended Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the database is
        /// hosted.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// The name of the Azure SQL Database Elastic Pool to be retrieved.
        /// </param>
        /// <param name='databaseName'>
        /// The name of the Azure SQL Database to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a Get Azure Sql Database request.
        /// </returns>
        Task<DatabaseGetResponse> GetDatabasesAsync(string resourceGroupName, string serverName, string recommendedElasticPoolName, string databaseName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about Azure SQL Recommended Elastic Pools.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Recommended
        /// Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server in which Azure SQL
        /// Recommended Elastic Pools are hosted.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool request.
        /// </returns>
        Task<RecommendedElasticPoolListResponse> ListAsync(string resourceGroupName, string serverName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about an Azure SQL Database inside of an Azure
        /// Sql Recommended Elastic Pool.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server on which the database is
        /// hosted.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// The name of the Azure SQL Recommended Elastic Pool to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Database request.
        /// </returns>
        Task<DatabaseListResponse> ListDatabasesAsync(string resourceGroupName, string serverName, string recommendedElasticPoolName, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about Azure SQL Recommended Elastic Pools.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the Azure SQL Recommended
        /// Serve belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server in which Azure SQL
        /// Recommended Elastic Pools are hosted.
        /// </param>
        /// <param name='expand'>
        /// The comma separated list of child objects that we want to expand on
        /// in response.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool request.
        /// </returns>
        Task<RecommendedElasticPoolListResponse> ListExpandedAsync(string resourceGroupName, string serverName, string expand, CancellationToken cancellationToken);
        
        /// <summary>
        /// Returns information about an recommended elastic pool metrics.
        /// </summary>
        /// <param name='resourceGroupName'>
        /// The name of the Resource Group to which the server belongs.
        /// </param>
        /// <param name='serverName'>
        /// The name of the Azure SQL Database Server.
        /// </param>
        /// <param name='recommendedElasticPoolName'>
        /// The name of the Azure SQL Recommended Elastic Pool to be retrieved.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// Represents the response to a List Azure Sql Recommended Elastic
        /// Pool metrics request.
        /// </returns>
        Task<RecommendedElasticPoolListMetricsResponse> ListMetricsAsync(string resourceGroupName, string serverName, string recommendedElasticPoolName, CancellationToken cancellationToken);
    }
}
